import { Component, inject, OnDestroy } from '@angular/core';
import { MatButtonModule } from '@angular/material/button';
import { MatIconModule } from '@angular/material/icon';
import { ActivatedRoute, RouterModule } from '@angular/router';
import { Group } from '../../shared/models/group';
import GroupService from '../../shared/services/group.service';
import { MatTableModule } from '@angular/material/table';
import { GroupListComponent } from "../group-list/group-list.component";
import { Policy } from '../../shared/models/policy';
import { MatDialog } from '@angular/material/dialog';
import { GroupItemEditComponent } from '../group-item-edit/group-item-edit.component';
import { MatMenuModule } from '@angular/material/menu';
import { HermesClientService } from '../../hermes-client.service';
import { GroupChatter } from '../../shared/models/group-chatter';
import { SpecialGroups } from '../../shared/utils/groups';
import { Permission } from '../../shared/models/permission';
import { Subscription } from 'rxjs';
import { PermissionService } from '../../shared/services/permission.service';
import PolicyService from '../../shared/services/policy.service';

@Component({
  selector: 'groups',
  imports: [
    MatButtonModule,
    MatIconModule,
    MatMenuModule,
    MatTableModule,
    RouterModule,
    GroupListComponent,
  ],
  templateUrl: './groups.component.html',
  styleUrl: './groups.component.scss'
})
export class GroupsComponent implements OnDestroy {
  private readonly _client = inject(HermesClientService);
  private readonly _route = inject(ActivatedRoute);
  private readonly _dialog = inject(MatDialog);
  private readonly _groupService = inject(GroupService);
  private readonly _permissionService = inject(PermissionService);
  private readonly _policyService = inject(PolicyService);
  private readonly subscriptions: (Subscription | undefined)[] = [];

  readonly specialGroups = SpecialGroups;

  private _groups: Group[] = [];
  private _chatters: GroupChatter[] = [];
  private _permissions: Permission[] = [];
  private _policies: Policy[] = [];

  opened = false;

  constructor() {
    this._route.data.subscribe(payload => {
      this._groups = payload['groups'];
      this._chatters = payload['chatters'];
      this._permissions = payload['permissions'];
      this._policies = payload['policies'];
    });

    this.subscriptions.push(this._permissionService.delete$?.subscribe(d => {
      if (d.error) {
        return;
      }

      this._permissionService.fetch().subscribe(permissions => this._permissions = permissions);
    }));

    this.subscriptions.push(this._groupService.deleteGroup$?.subscribe(d => {
      if (d.error) {
        return;
      }

      this._groupService.fetch().subscribe(data => this._groups = data.groups);
    }));

    this.subscriptions.push(this._groupService.deleteChatter$?.subscribe(d => {
      if (d.error) {
        return;
      }

      this._groupService.fetch().subscribe(data => this._chatters = data.chatters);
    }));

    this.subscriptions.push(this._policyService.delete$?.subscribe(d => {
      if (d.error) {
        return;
      }

      this._policyService.fetch().subscribe(policies => this._policies = policies);
    }));
  }

  ngOnDestroy(): void {
    if (this.subscriptions) {
      for (let subscription of this.subscriptions) {
        if (subscription)
          subscription.unsubscribe();
      }
    }
  }

  get groups() {
    return this._groups;
  }

  get chatters() {
    return this._chatters;
  }

  get permissions() {
    return this._permissions;
  }

  get policies() {
    return this._policies;
  }

  openDialog(groupName: string): void {
    if (this.opened) {
      return;
    }

    this.opened = true;
    const dialogRef = this._dialog.open(GroupItemEditComponent, {
      data: { group: { id: '', user_id: '', name: groupName, priority: 0 }, isSpecial: groupName.length > 0 },
    });

    dialogRef.afterClosed().subscribe((result: Group | undefined) => this.opened = false);
  }

  compare(a: Group, b: Group) {
    return a.name.toLowerCase().localeCompare(b.name.toLowerCase());
  }

  exists(groupName: string) {
    return this._groups.some(g => g.name == groupName);
  }
}